package Game.Entity
{
	public class Ship extends GameEntity
	{
		public var x:Number;
		public var y:Number;
		public var deltaX:Number;
		public var deltaY:Number;
		public var speed:Number;
		public var rotation:Number;
		public var energy:Number;
		
		public var team:int;
		public var target:Node;
		
		public var trailTimer:Number;
		public var lastTrail:Trail;
		
		public function Ship()
		{
		}
		
		public function initShip(node:Node, target:Node):void {
			super.init(node.game);
			this.x = node.x;
			this.y = node.y;
			this.target = target;
			this.team = node.team;
			this.energy = 0.04;
			speed = 150;
			
			trailTimer = 0;
			lastTrail = null;
			
			rotation = Math.random()*Math.PI*2;
			deltaX = Math.cos(rotation)*(100+Math.random()*100);
			deltaY = Math.sin(rotation)*(100+Math.random()*100);
		}
		
		public override function update(dt:Number):void {
			var dx:Number = target.x - x;
			var dy:Number = target.y - y;
			var dist:Number = Math.sqrt(dx*dx + dy*dy);
			
			if(dist > target.size*50+4) {
				x += deltaX*dt;
				y += deltaY*dt;
				deltaX -= deltaX*dt*1.2;
				deltaY -= deltaY*dt*1.2;
				
				var angle:Number = Math.atan2(dy, dx);
				deltaX += Math.cos(angle)*speed*dt;
				deltaY += Math.sin(angle)*speed*dt;
			} else {
				target.hit(this);
				active = false;
			}
			
			if(lastTrail) {
				dx = this.x - lastTrail.x;
				dy = this.y - lastTrail.y;
				dist = Math.sqrt(dx*dx + dy*dy);
				angle = Math.atan2(dy, dx);
				lastTrail.width = dist;
				lastTrail.rotation = angle;
			}
			
			trailTimer -= dt;
			if(trailTimer <= 0) {
				lastTrail = game.addTrail(x, y, game.colors[team]);
				trailTimer = 0.1;
			}
		}
	}
}