package Game
{
	import Game.Entity.AI;
	import Game.Entity.EntityPool;
	import Game.Entity.Node;
	import Game.Entity.Ship;
	import Game.Entity.Trail;
	
	import starling.animation.Juggler;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.QuadBatch;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	
	public class GameScene extends Sprite
	{	
		public var cover:Quad;
		public var ui:GameUI;
		public var pauseMenu:PauseMenu;
		public var juggler:Juggler;
		
		public var entities:Array;
		public var nodes:EntityPool;
		public var ships:EntityPool;
		public var trails:EntityPool;
		public var ais:EntityPool;
		
		public var gameLayer:Sprite;
		public var batch:QuadBatch;
		public var glowBatch:QuadBatch;
		public var shipBatch:QuadBatch;
		public var trailBatch:QuadBatch;
		public var uiBatch:QuadBatch;
		
		public var gameOver:Boolean;
		public var paused:Boolean;
		
		public var pauseBtn:UIButton;
		public var tutorial:Image;
		
		public var colors:Array = [0x3b303e, 0x0065c3, 0xb70027, 0x248a00, 0xb76700];
		public var uiColors:Array = [0x3b303e, 0x43a0ff, 0xfe3f71, 0x68c647, 0xf0993d];
		
		public function GameScene()
		{
			ui = new GameUI(this);
			juggler = new Juggler();
			
			nodes = new EntityPool();
			ships = new EntityPool();
			trails = new EntityPool();
			ais = new EntityPool();
			entities = [nodes, ships, trails, ais];
			
			addChild(new Image(Root.assets.getTexture("bg03")));
			
			gameLayer = new Sprite();
			addChild(gameLayer);
			
			batch = new QuadBatch();
			gameLayer.addChild(batch);
			
			glowBatch = new QuadBatch();
			gameLayer.addChild(glowBatch);
			
			trailBatch = new QuadBatch();
			gameLayer.addChild(trailBatch);
			
			shipBatch = new QuadBatch();
			gameLayer.addChild(shipBatch);
			
			gameLayer.blendMode = "add";
			
			uiBatch = new QuadBatch();
			addChild(uiBatch);
			
			tutorial = new Image(Root.assets.getTexture("tutorial"));
			tutorial.x = Constants.STAGE_WIDTH*0.5 - tutorial.width*0.5 - 8;
			tutorial.y = 420;
			addChild(tutorial);
			
			cover = new Quad(800, 600, 0xffffff);
			cover.blendMode = "add";
			cover.alpha = 0;
			addChild(cover);
			
			pauseMenu = new PauseMenu(this);
			pauseMenu.visible = false;
			
			pauseBtn = new UIButton("btn_pause");
			pauseBtn.x = 12;
			pauseBtn.y = 12;
			addChild(pauseBtn);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// INITIALIZERS
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		public function init():void {
			ui.init();
			addChild(pauseBtn);
			addChild(pauseMenu);
			//addChild(pauseBtn);
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].init(this);
			//hero.init(this);
			
			gameOver = true;
			tutorial.visible = false;
			cover.alpha = 0;
			
			initNodes(Globals.levelData[Globals.level][0]);
			initTeams(Globals.levelData[Globals.level][1], Globals.levelData[Globals.level][2]);
			
			animateIn();
			start();
		}
		
		public function initNodes(num:int):void {
			var l:int = num;
			for(var i:int = 0; i < l; i++) {
				var x:Number = Math.random()*700 + 50;
				var y:Number = Math.random()*500 + 50;
				if(Globals.level == 0) {
					y = Math.random()*250 + 100;
					tutorial.visible = true;
				}
				var size:Number = Math.random()*0.5+0.3;
				var overflow:int = 0;
				while(overlap(x, y) && overflow < 1000) {
					x = Math.random()*700 + 50;
					y = Math.random()*500 + 50;
					if(Globals.level == 0) y = Math.random()*250 + 100;
					overflow++;
				}
				if(overflow < 1000) addNode(x, y, size);
			}
		}
		
		public function initTeams(num:int, delay:Number):void {
			var sorter:Array = [];
			var sortNode:NodeSorter;
			var dx:Number, dy:Number; 
			
			if(num == 2) {
				var l:int = nodes.active.length;
				for(var i:int = 0; i < l; i++) {
					var node:Node = nodes.active[i];
					sortNode = new NodeSorter(node);
					dx = node.x - 0;
					dy = 0;
					sortNode.blDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - Constants.STAGE_WIDTH;
					dy = 0;
					sortNode.trDist = Math.sqrt(dx*dx + dy*dy);
					sorter.push(sortNode);
				}
				
				sorter.sortOn("blDist", Array.NUMERIC);
				sorter[0].node.team = 1;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("trDist", Array.NUMERIC);
				sorter[0].node.team = 2;
				sorter[0].node.setSize(0.5);
				
				addAI(2, delay);
			} else if(num == 3) {
				l = nodes.active.length;
				for(i = 0; i < l; i++) {
					node = nodes.active[i];
					sortNode = new NodeSorter(node);
					dx = node.x - 0;
					dy = node.y - Constants.STAGE_HEIGHT;
					sortNode.blDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - Constants.STAGE_WIDTH;
					dy = node.y - 0;
					sortNode.trDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - Constants.STAGE_WIDTH*0.5;
					dy = node.y - Constants.STAGE_HEIGHT*0.5;
					sortNode.midDist = Math.sqrt(dx*dx + dy*dy);
					sorter.push(sortNode);
				}
				
				sorter.sortOn("blDist", Array.NUMERIC);
				sorter[0].node.team = 1;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("trDist", Array.NUMERIC);
				sorter[0].node.team = 2;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("midDist", Array.NUMERIC);
				sorter[0].node.team = 3;
				sorter[0].node.setSize(0.5);
				
				addAI(2, delay);
				addAI(3, delay);
			} else if(num == 4) {
				l = nodes.active.length;
				for(i = 0; i < l; i++) {
					node = nodes.active[i];
					sortNode = new NodeSorter(node);
					dx = node.x - 0;
					dy = node.y - 0;
					sortNode.tlDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - 0;
					dy = node.y - Constants.STAGE_HEIGHT;
					sortNode.blDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - Constants.STAGE_WIDTH;
					dy = node.y - 0;
					sortNode.trDist = Math.sqrt(dx*dx + dy*dy);
					dx = node.x - Constants.STAGE_WIDTH;
					dy = node.y - Constants.STAGE_HEIGHT;
					sortNode.brDist = Math.sqrt(dx*dx + dy*dy);
					sorter.push(sortNode);
				}
				
				sorter.sortOn("blDist", Array.NUMERIC);
				sorter[0].node.team = 1;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("tlDist", Array.NUMERIC);
				sorter[0].node.team = 2;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("trDist", Array.NUMERIC);
				sorter[0].node.team = 3;
				sorter[0].node.setSize(0.5);
				
				sorter.sortOn("brDist", Array.NUMERIC);
				sorter[0].node.team = 4;
				sorter[0].node.setSize(0.5);
				
				addAI(2, delay);
				addAI(3, delay);
				addAI(4, delay);
			}
		}
		
		public function overlap(x:Number, y:Number):Boolean {
			var l:int = nodes.active.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes.active[i];
				var dx:Number = node.x - x;
				var dy:Number = node.y - y;
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
				if(dist < 100) return true;
			}
			return false;
		}
		
		public function deInit():void {
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].deInit();
			
			pauseBtn.deInit();
			pauseBtn.removeEventListener("clicked", on_pause);
			removeEventListener(Event.ENTER_FRAME, update);
		}
		
		public function animateIn():void {
			alpha = 0;
			juggler.tween(this, 0.5, {alpha:1});
		}
		
		public function animateOut():void {
			juggler.tween(this, 0.5, {alpha:0});
		}
		
		public function start():void {
			gameOver = false;
			pauseBtn.init();
			pauseBtn.addEventListener("clicked", on_pause);
			addEventListener(Event.ENTER_FRAME, update);
			//GS.playMusic();
		}
		
		public function reset():void {
			gameOver = true;
			cover.alpha = 0;
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].init(this);
			deInit();
		}
		
		public function quit():void {
			animateOut();
			juggler.delayCall(function():void{dispatchEventWith("quit");}, 0.5);
		}
		
		public function on_pause(e:Event):void {
			//Globals.main.on_deactivate(null);
			pauseMenu.show();
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// UPDATES
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		public function update(e:EnterFrameEvent):void {
			if(pauseMenu.visible) return;
			var dt:Number = e.passedTime;
			juggler.advanceTime(dt);
			dt *= 0.5;
			
			ui.update(dt);
			
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].update(dt);
			
			batch.reset();
			glowBatch.reset();
			shipBatch.reset();
			trailBatch.reset();
			uiBatch.reset();
			drawNodes();
			drawShips();
			drawTrails();
			drawMeter();
			
			if(!gameOver) checkGameOver();
			
			GS.update(dt);
		}
		
		public function drawNodes():void {
			var l:int = nodes.active.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes.active[i];
				var size:Number = node.size*node.energy*50;
				if(size < 0) size = 0;
				if(node.team == 0) {
					drawCircle(node.x, node.y, node.size*50+4, node.size*50, colors[0]);
					if(node.captureTeam != 0) {
						var minSize:Number = node.size*50 - size;
						drawCircle(node.x, node.y, node.size*50, minSize, colors[node.captureTeam]);
						drawGlowCircle(node.x, node.y, size, 0, colors[node.captureTeam]);
					}
				} else {
					drawCircle(node.x, node.y, node.size*50+4, node.size*50, colors[node.team]);
					drawCircle(node.x, node.y, size, 0, colors[node.team]);
					drawGlowCircle(node.x, node.y, size+25, 0, colors[node.team]);
				}
				
				if(node.selected || node == ui.hoverNode) drawCircle(node.x, node.y, node.size*50 + 8, node.size*50 + 5, 0xffffff);
				
				if(node.selected) {
					var dx:Number = ui.mouseLoc.x - node.x;
					var dy:Number = ui.mouseLoc.y - node.y;
					var dist:Number = Math.sqrt(dx*dx + dy*dy);
					if(dist < node.size*50+5) continue;
					var angle:Number = Math.atan2(dy, dx);
					var nodeX:Number = node.x+Math.cos(angle)*(node.size*50+7);
					var nodeY:Number = node.y+Math.sin(angle)*(node.size*50+7);
					if(ui.hoverNode) {
						if(node == ui.hoverNode) continue;
						dx = node.x - ui.hoverNode.x;
						dy = node.y - ui.hoverNode.y;
						angle = Math.atan2(dy, dx);
						nodeX = node.x+Math.cos(angle+Math.PI)*(node.size*50+7);
						nodeY = node.y+Math.sin(angle+Math.PI)*(node.size*50+7);
						var targetX:Number = ui.hoverNode.x+Math.cos(angle)*(ui.hoverNode.size*50+7);
						var targetY:Number = ui.hoverNode.y+Math.sin(angle)*(ui.hoverNode.size*50+7);
						drawLine(nodeX, nodeY, targetX, targetY, 0xffffff, 2);
					} else {
						drawLine(nodeX, nodeY, ui.mouseLoc.x, ui.mouseLoc.y, 0xffffff, 2);
					}
					
				}
			}
		}
		
		public function drawShips():void {
			var l:int = ships.active.length;
			for(var i:int = 0; i < l; i++) {
				var ship:Ship = ships.active[i];
				drawShip(ship.x, ship.y, 2, 0, colors[ship.team]);
			}
		}
		
		public function drawTrails():void {
			var l:int = trails.active.length;
			for(var i:int = 0; i < l; i++) {
				var trail:Trail = trails.active[i];
				drawTrail(trail.x, trail.y, trail.width, trail.rotation, trail.color, 2, trail.alpha);
			}
		}
		
		public var teamStats:Array = [0, 0, 0, 0, 0];
		public function drawMeter():void {
			teamStats[0] = 0;
			teamStats[1] = 0;
			teamStats[2] = 0;
			teamStats[3] = 0;
			teamStats[4] = 0;
			
			var l:int = nodes.active.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes.active[i];
				if(node.team > 0) teamStats[node.team] += node.size;
			}
			
			var total:Number = teamStats[1] + teamStats[2] + teamStats[3] + teamStats[4];
			var team1Width:Number = (teamStats[1]/total) * 800;
			var team2Width:Number = (teamStats[2]/total) * 800;
			var team3Width:Number = (teamStats[3]/total) * 800;
			var team4Width:Number = (teamStats[4]/total) * 800;
			
			var thickness:Number = 6;
			var _y:Number = Constants.STAGE_HEIGHT - thickness*0.5;
			drawUILine(0, _y, team1Width, _y, uiColors[1], thickness);
			drawUILine(team1Width, _y, team1Width+team2Width, _y, uiColors[2], thickness);
			drawUILine(team1Width+team2Width, _y, team1Width+team2Width+team3Width, _y, uiColors[3], thickness);
			drawUILine(team1Width+team2Width+team3Width, _y, team1Width+team2Width+team3Width+team4Width, _y, uiColors[4], thickness);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// GAME LOGIC
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		public var activeTeams:Array = [];
		public function checkGameOver():void {
			activeTeams.length = 0;
			
			var l:int = nodes.active.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes.active[i];
				if(node.team > 0 && activeTeams.indexOf(node.team) == -1) activeTeams.push(node.team);
			}
			
			l = ships.active.length;
			for(i = 0; i < l; i++) {
				var ship:Ship = ships.active[i];
				if(ship.team > 0 && activeTeams.indexOf(ship.team) == -1) activeTeams.push(node.team);
			}
			
			if(activeTeams.length == 1) {
				gameOver = true;
				var winTeam:int = activeTeams[0];
				cover.color = colors[winTeam];
				juggler.tween(cover, 0.1, {alpha:0.5});
				juggler.tween(cover, 2.0, {alpha:0, delay:0.5});
				if(winTeam == 1) {
					Globals.levelReached = Globals.level + 1;
				} else {
					
				}
				juggler.delayCall(quit, 2.5);
			}
		}
		
		public function sendShips(node:Node, target:Node):int {
			if(node == target) return 0;
			var num:int = node.size*node.energy*50;
			for(var j:int = 0; j < num; j++) {
				addShip(node, target);
			}
			node.energy = 0;
			GS.playPing02();
			
			return num;
		}
		
		public function addNode(x:Number, y:Number, size:Number):Node {
			var node:Node = nodes.getReserve() as Node;
			if(!node) node = new Node();
			
			node.initNode(this, x, y, size);
			nodes.addEntity(node);
			return node;
		}
		
		public function addShip(node:Node, target:Node):Ship {
			var ship:Ship = ships.getReserve() as Ship;
			if(!ship) ship = new Ship();
			
			ship.initShip(node, target);
			ships.addEntity(ship);
			return ship;
		}
		
		public function addTrail(x:Number, y:Number, color:uint):Trail {
			var trail:Trail = trails.getReserve() as Trail;
			if(!trail) trail = new Trail();
			
			trail.initTrail(this, x, y, color);
			trails.addEntity(trail);
			return trail;
		}
		
		public function addAI(team:int, delay:Number):AI {
			var ai:AI = ais.getReserve() as AI;
			if(!ai) ai = new AI();
			
			ai.initAI(this, team, delay);
			ais.addEntity(ai);
			return ai;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// DRAWING FUNCITONS
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		public function getWhiteColor(val:Number):uint {
			if(val <= 0) return 0xda0000
			var r:uint = 0xda + ((0xff-0xda)*val);
			var g:uint = 0x00 + (0xff * val);
			var b:uint = 0x00 + (0xff * val);
			return ( ( r << 16 ) | ( g << 8 ) | b );
		}
		
		public var drawQuad:Quad;
		public function drawCircle(x:Number, y:Number, outer_radius:Number, inner_radius:Number, color:uint, alpha:Number=1.0, segments:int=32):void {
			if(!drawQuad) drawQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = drawQuad;
			quad.color = color;
			quad.alpha = alpha;
			
			var l:int = segments;
			var step:Number = Math.PI*2/l;
			var angle:Number = 0;
			var x1:Number = x+Math.cos(angle)*outer_radius;
			var y1:Number = y+Math.sin(angle)*outer_radius;
			var x2:Number = x+Math.cos(angle)*inner_radius;
			var y2:Number = y+Math.sin(angle)*inner_radius;
			for(var i:int = 0; i < l; i++) {
				quad.setVertexPosition(0, x1, y1);
				quad.setVertexPosition(2, x2, y2);
				angle += step;
				x1 = x+Math.cos(angle)*outer_radius;
				y1 = y+Math.sin(angle)*outer_radius;
				x2 = x+Math.cos(angle)*inner_radius;
				y2 = y+Math.sin(angle)*inner_radius;
				quad.setVertexPosition(1, x1, y1);
				quad.setVertexPosition(3, x2, y2);
				batch.addQuad(quad);
			}
		}
		
		public function drawShip(x:Number, y:Number, outer_radius:Number, inner_radius:Number, color:uint, segments:int=4):void {
			if(!drawQuad) drawQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = drawQuad;
			quad.color = color;
			quad.alpha = alpha;
			
			var l:int = segments;
			var step:Number = Math.PI*2/l;
			var angle:Number = 0;
			var x1:Number = x+Math.cos(angle)*outer_radius;
			var y1:Number = y+Math.sin(angle)*outer_radius;
			var x2:Number = x+Math.cos(angle)*inner_radius;
			var y2:Number = y+Math.sin(angle)*inner_radius;
			for(var i:int = 0; i < l; i++) {
				quad.setVertexPosition(0, x1, y1);
				quad.setVertexPosition(2, x2, y2);
				angle += step;
				x1 = x+Math.cos(angle)*outer_radius;
				y1 = y+Math.sin(angle)*outer_radius;
				x2 = x+Math.cos(angle)*inner_radius;
				y2 = y+Math.sin(angle)*inner_radius;
				quad.setVertexPosition(1, x1, y1);
				quad.setVertexPosition(3, x2, y2);
				shipBatch.addQuad(quad);
			}
		}
		
		public var glowQuad:Quad;
		public function drawGlowCircle(x:Number, y:Number, outer_radius:Number, inner_radius:Number, color:uint, segments:int=32):void {
			if(!glowQuad) {
				glowQuad = new Quad(10, 10, 0xffffff);
				glowQuad.alpha = 0.5;
				glowQuad.setVertexAlpha(0, 0);
				glowQuad.setVertexAlpha(1, 0);
			}
			var quad:Quad = glowQuad;
			quad.color = color;
			
			var l:int = segments;
			var step:Number = Math.PI*2/l;
			var angle:Number = 0;
			var x1:Number = x+Math.cos(angle)*outer_radius;
			var y1:Number = y+Math.sin(angle)*outer_radius;
			var x2:Number = x+Math.cos(angle)*inner_radius;
			var y2:Number = y+Math.sin(angle)*inner_radius;
			for(var i:int = 0; i < l; i++) {
				quad.setVertexPosition(0, x1, y1);
				quad.setVertexPosition(2, x2, y2);
				angle += step;
				x1 = x+Math.cos(angle)*outer_radius;
				y1 = y+Math.sin(angle)*outer_radius;
				x2 = x+Math.cos(angle)*inner_radius;
				y2 = y+Math.sin(angle)*inner_radius;
				quad.setVertexPosition(1, x1, y1);
				quad.setVertexPosition(3, x2, y2);
				glowBatch.addQuad(quad);
			}
		}
		
		public var lineQuad:Quad;
		public function drawLine(x1:Number, y1:Number, x2:Number, y2:Number, color:uint, thickness:Number=2, alpha:Number=1):void {
			if(!lineQuad) lineQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = lineQuad;
			quad.color = color;
			quad.alpha = alpha;
			quad.rotation = 0;
			thickness *= 0.5;
			
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			var angle:Number = Math.atan2(dy, dx);
			var dist:Number = Math.sqrt(dx*dx +dy*dy);
			quad.x = x1;
			quad.y = y1;
			quad.setVertexPosition(0, 0, -thickness);
			quad.setVertexPosition(1, dist, -thickness);
			quad.setVertexPosition(2, 0, thickness);
			quad.setVertexPosition(3, dist, thickness);
			quad.rotation = angle;
			batch.addQuad(quad);
		}
		
		public function drawUILine(x1:Number, y1:Number, x2:Number, y2:Number, color:uint, thickness:Number=2, alpha:Number=1):void {
			if(!lineQuad) lineQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = lineQuad;
			quad.color = color;
			quad.alpha = alpha;
			quad.rotation = 0;
			thickness *= 0.5;
			
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			var angle:Number = Math.atan2(dy, dx);
			var dist:Number = Math.sqrt(dx*dx +dy*dy);
			quad.x = x1;
			quad.y = y1;
			quad.setVertexPosition(0, 0, -thickness);
			quad.setVertexPosition(1, dist, -thickness);
			quad.setVertexPosition(2, 0, thickness);
			quad.setVertexPosition(3, dist, thickness);
			quad.rotation = angle;
			uiBatch.addQuad(quad);
		}
		
		public function drawTrail(x1:Number, y1:Number, dist:Number, angle:Number, color:uint, thickness:Number=2, alpha:Number=1):void {
			if(!lineQuad) lineQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = lineQuad;
			quad.color = color;
			quad.alpha = alpha;
			quad.rotation = 0;
			thickness *= 0.5;
			
			quad.x = x1;
			quad.y = y1;
			quad.setVertexPosition(0, 0, -thickness);
			quad.setVertexPosition(1, dist, -thickness);
			quad.setVertexPosition(2, 0, thickness);
			quad.setVertexPosition(3, dist, thickness);
			quad.rotation = angle;
			trailBatch.addQuad(quad);
		}
	}
}