package Game
{
	import flash.geom.Point;
	
	import Game.Entity.Node;
	
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;

	public class GameUI extends Sprite
	{
		public var game:GameScene;
		public var hitQuad:Quad;
		public var mouseLoc:Point;
		public var mousePresent:Boolean;
		public var mouseDown:Boolean;
		
		public var hoverNode:Node;
		
		public function GameUI(game:GameScene)
		{
			this.game = game;
			mouseLoc = new Point(0, 0);
			
			hitQuad = new Quad(Constants.STAGE_WIDTH, Constants.STAGE_HEIGHT, 0x0000ff);
			hitQuad.alpha = 0;
			addChild(hitQuad);
		}
		
		public function init():void {
			game.addChild(this);
			addEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		public function start():void {
			
		}
		
		public function deInit():void {
			game.removeChild(this);
			removeEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		public function on_touch(e:TouchEvent):void {
			if(game.pauseMenu.visible) return;
			var touch:Touch = e.getTouch(this);
			if(!touch) {
				mousePresent = false;
				return;
			}
			
			mousePresent = true;
			switch(touch.phase) {
				case TouchPhase.HOVER:
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.BEGAN:
					mouseDown = true;
					touch.getLocation(this, mouseLoc);
					var node:Node = getClosestNode(mouseLoc.x, mouseLoc.y);
					if(node && (node.team == 1 || node.captureTeam == 1)) node.selected = true;
					break;
				case TouchPhase.MOVED:
					touch.getLocation(this, mouseLoc);
					node = getClosestNode(mouseLoc.x, mouseLoc.y);
					if(node && (node.team == 1 || node.captureTeam == 1)) node.selected = true;
					break;
				case TouchPhase.ENDED:
					mouseDown = false;
					touch.getLocation(this, mouseLoc);
					node = getClosestNode(mouseLoc.x, mouseLoc.y);
					sendShips();
					//clearSelection();
					break;
			}
		}
		
		public function update(dt:Number):void {
			hoverNode = getClosestNode(mouseLoc.x, mouseLoc.y);
		}
		
		public function getClosestNode(x:Number, y:Number):Node {
			var closest:Node;
			var min:Number = 70;
			
			var nodes:Array = game.nodes.active;
			var l:int = nodes.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes[i];
				var dx:Number = node.x - x;
				var dy:Number = node.y - y;
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
				if(dist < node.size*50+20 && dist < min) {
					min = dist;
					closest = node;
				}
			}
			
			return closest;
		}
		
		public function sendShips():void {
			var nodes:Array = game.nodes.active;
			var l:int = nodes.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes[i];
				if(node.team == 1 && node.selected && hoverNode) {
					game.sendShips(node, hoverNode);
				}
				node.selected = false;
			}
		}
		
		public function clearSelection():void {
			var nodes:Array = game.nodes.active;
			var l:int = nodes.length;
			for(var i:int = 0; i < l; i++) {
				var node:Node = nodes[i];
				node.selected = false;
			}
		}
	}
}