package Menus
{
	import flash.geom.Point;
	
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	
	public class TitleMenu extends Sprite
	{
		public var buttons:Array;
		
		public var hitQuad:Quad;
		public var mouseLoc:Point;
		public var mousePresent:Boolean;
		public var mouseDown:Boolean;
		
		public function TitleMenu()
		{
			addChild(new Image(Root.assets.getTexture("bg03")));
			
			var title:Image = new Image(Root.assets.getTexture("title"));
			title.x = Constants.STAGE_WIDTH*0.5 - title.width*0.5;
			title.y = 80;
			addChild(title);
			
			var credits:Image = new Image(Root.assets.getTexture("credits"));
			credits.x = Constants.STAGE_WIDTH*0.5 - credits.width*0.5;
			credits.y = 525;
			addChild(credits);
			
			buttons = new Array();
			var l:int = 12;
			for(var i:int = 0; i < l; i++) {
				var btn:TitleButton = new TitleButton();
				btn.setNum(i+1);
				btn.x = 250 + (i%4)*(100);
				btn.y = 220 + int(i/4)*100;
				addChild(btn);
				buttons.push(btn);
			}
			
			mouseLoc = new Point(0, 0);
			hitQuad = new Quad(Constants.STAGE_WIDTH, Constants.STAGE_HEIGHT, 0xff0000);
			hitQuad.alpha = 0;
			addChild(hitQuad);
		}
		
		public function init():void {
			setButtons();
			mouseLoc.x = mouseLoc.y = 0;
			addEventListener(TouchEvent.TOUCH, on_touch);
			addEventListener(Event.ENTER_FRAME, update);
		}
		
		public function deInit():void {
			removeEventListener(TouchEvent.TOUCH, on_touch);
			removeEventListener(Event.ENTER_FRAME, update);
		}
		
		public function update(e:EnterFrameEvent):void {
			setButtons();
			var btn:TitleButton = getClosestButton(mouseLoc.x, mouseLoc.y);
			if(btn) {
				var index:int = buttons.indexOf(btn);
				if(Globals.levelReached >= index) btn.alpha = 1.0;
			}
		}
		
		public function on_touch(e:TouchEvent):void {
			var touch:Touch = e.getTouch(this);
			if(!touch) {
				mousePresent = false;
				return;
			}
			
			mousePresent = true;
			switch(touch.phase) {
				case TouchPhase.HOVER:
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.BEGAN:
					mouseDown = true;
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.MOVED:
					touch.getLocation(this, mouseLoc);
					break;
				case TouchPhase.ENDED:
					mouseDown = false;
					touch.getLocation(this, mouseLoc);
					startLevel();
					break;
			}
		}
		
		public function startLevel():void {
			var btn:TitleButton = getClosestButton(mouseLoc.x, mouseLoc.y);
			if(btn) {
				var index:int = buttons.indexOf(btn);
				if(Globals.levelReached >= index) {
					Globals.level = buttons.indexOf(btn);
					dispatchEventWith("start");
					GS.playPing01();
				}
			}
		}
		
		public function setButtons():void {
			var l:int = buttons.length;
			for(var i:int = 0; i < l; i++) {
				var btn:TitleButton = buttons[i];
				if(Globals.levelReached >= i) {
					btn.alpha = 0.5;
					if(Globals.levelReached > i) btn.setColor(0x57b0ff);
				} else btn.alpha = 0.25;
			}
		}
		
		public function getClosestButton(x:Number, y:Number):TitleButton {
			var closest:TitleButton;
			var min:Number = 30;
			
			var l:int = buttons.length;
			for(var i:int = 0; i < l; i++) {
				var btn:TitleButton = buttons[i];
				var dx:Number = btn.x - x;
				var dy:Number = btn.y - y;
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
				if(dist < min) {
					min = dist;
					closest = btn;
				}
			}
			
			return closest;
		}
	}
}